use chrono::NaiveDate;
use diesel::prelude::*;
use serde::{Deserialize, Serialize};

use crate::{
    db::{schema::*, DbPool, DbResult, Pagination, Plant},
    server::query::{Query, ToQuery},
};

#[derive(Serialize, Queryable, Selectable)]
#[diesel(table_name = images)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Image {
    pub id: i32,
    pub plant_id: i32,
    pub date_taken: NaiveDate,
    pub mime_type: String,
    pub note: Option<String>,
}

#[derive(Deserialize, Insertable)]
#[diesel(table_name = images)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct NewImage {
    plant_id: i32,
    date_taken: NaiveDate,
    mime_type: String,
    note: Option<String>,
}

#[derive(Deserialize, Clone, Copy)]
pub struct ImageFilter {
    pub plant_id: Option<i32>,
}

impl ToQuery for ImageFilter {
    fn to_query(self) -> crate::server::query::Query {
        let mut out = vec![];

        if let Some(plant_id) = self.plant_id {
            out.push(("plant_id".to_string(), plant_id.to_string()));
        }

        Query(out)
    }
}

impl NewImage {
    pub fn new(
        plant_id: i32,
        date_taken: NaiveDate,
        mime_type: String,
        note: Option<String>,
    ) -> Self {
        Self {
            plant_id,
            date_taken,
            mime_type,
            note,
        }
    }

    pub fn insert(self, pool: &DbPool) -> DbResult<Image> {
        Ok(self
            .insert_into(images::table)
            .returning(Image::as_returning())
            .get_result(&mut pool.get()?)?)
    }
}

impl Image {
    pub fn by_id(pool: &DbPool, id: i32) -> DbResult<Option<Self>> {
        Ok(images::table
            .find(id)
            .select(Self::as_select())
            .first(&mut pool.get()?)
            .optional()?)
    }

    pub fn page_with_plants(
        pool: &DbPool,
        page: Pagination,
        filter: ImageFilter,
    ) -> DbResult<Vec<(Self, Plant)>> {
        let mut query = images::table
            .inner_join(plants::table)
            .select((Self::as_select(), Plant::as_select()))
            .into_boxed();

        if let Some(plant_id) = filter.plant_id {
            query = query.filter(images::dsl::plant_id.eq(plant_id));
        }

        Ok(query
            .order_by((images::dsl::date_taken.desc(), images::dsl::id.desc()))
            .offset((page.page * page.per_page).into())
            .limit(page.per_page.into())
            .get_results(&mut pool.get()?)?)
    }
}
