use serde::Serialize;
use tera::Context;

use crate::{
    db::Pagination,
    server::query::{Query, ToQuery},
};

use super::Template;

#[derive(Clone, Copy)]
pub enum List {
    Ul,
    Card,
}

pub enum ListItem {
    Plant,
}

pub enum ListCard {
    Image,
}

impl Template for List {
    fn template(&self) -> &'static str {
        match self {
            List::Ul => "list/ul.html",
            List::Card => "list/card.html",
        }
    }
}

impl Template for ListItem {
    fn template(&self) -> &'static str {
        match self {
            Self::Plant => "li/plant.html",
        }
    }
}

impl Template for ListCard {
    fn template(&self) -> &'static str {
        match self {
            Self::Image => "card/image.html",
        }
    }
}

impl List {
    pub fn items<T: Template, I: Serialize>(
        self,
        heading: &'static str,
        item_tmpl: T,
        items: Vec<I>,
    ) -> ListWrapper<T, I> {
        ListWrapper {
            list: self,
            heading,
            item_tmpl,
            items,
            next_page: None,
            query: None,
        }
    }
}

pub struct ListWrapper<T: Template, I: Serialize> {
    list: List,
    heading: &'static str,
    item_tmpl: T,
    items: Vec<I>,
    next_page: Option<(&'static str, Pagination)>,
    query: Option<Query>,
}

impl<T: Template, I: Serialize> ListWrapper<T, I> {
    pub fn page(mut self, url: &'static str, mut page: Pagination) -> Self {
        if self.items.len() as u32 == page.per_page {
            page.page += 1;

            self.next_page = Some((url, page));
        }

        self
    }

    pub fn query(mut self, query: impl ToQuery) -> Self {
        if let Some(old_query) = self.query {
            self.query = Some(old_query.join(query));
        } else {
            self.query = Some(query.to_query());
        }

        self
    }
}

impl<T: Template, I: Serialize> Template for ListWrapper<T, I> {
    fn template(&self) -> &'static str {
        self.list.template()
    }

    fn render(&self, tera: &tera::Tera) -> tera::Result<String> {
        let mut ctx = Context::new();

        // Render each item separately
        let items = self
            .items
            .iter()
            .map(|item| {
                ctx.insert("item", &item);

                tera.render(self.item_tmpl.template(), &ctx)
            })
            .collect::<tera::Result<Vec<String>>>()?;

        ctx.remove("item");
        ctx.insert("heading", &self.heading);
        ctx.insert("items", &items);

        if let Some((url, page)) = &self.next_page {
            let query = if let Some(query) = &self.query {
                query.clone().join(page.clone())
            } else {
                page.to_query()
            };

            let full_url = format!("{url}?{}", query.encode());
            ctx.insert("next_page_url", &full_url);
        }

        self.render_ctx(tera, &ctx)
    }
}
